package com.zrshuiwu.common.core.util;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.LocalDateTimeUtil;
import java.sql.Timestamp;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Date;

public class DateUtils {

   public static final String YEAR_MONTH_DAY = "yyyyMMdd";
   static Integer[] minute15Array = new Integer[]{Integer.valueOf(0), Integer.valueOf(15), Integer.valueOf(30), Integer.valueOf(45), Integer.valueOf(60)};
   static Integer[] minute30Array = new Integer[]{Integer.valueOf(0), Integer.valueOf(30), Integer.valueOf(60)};
   static Integer[] minute5Array = new Integer[]{Integer.valueOf(0), Integer.valueOf(5), Integer.valueOf(10), Integer.valueOf(15), Integer.valueOf(20), Integer.valueOf(25), Integer.valueOf(30), Integer.valueOf(35), Integer.valueOf(40), Integer.valueOf(45), Integer.valueOf(50), Integer.valueOf(55), Integer.valueOf(60)};
   static Integer[] minute10Array = new Integer[]{Integer.valueOf(0), Integer.valueOf(10), Integer.valueOf(20), Integer.valueOf(30), Integer.valueOf(40), Integer.valueOf(50), Integer.valueOf(60)};


   public static Date toDate(LocalDateTime localDateTime) {
      return Date.from(localDateTime.atZone(ZoneId.systemDefault()).toInstant());
   }

   public static LocalDateTime toLocalDateTime(Date date) {
      return LocalDateTime.ofInstant(date.toInstant(), ZoneId.systemDefault());
   }

   public static LocalDateTime beginOfMonth(LocalDateTime localDateTime) {
      String dateTime = localDateTime.format(DateTimeFormatter.ofPattern("yyyy-MM"));
      dateTime = dateTime + "-01 00:00:00";
      return LocalDateTimeUtil.parse(dateTime, "yyyy-MM-dd HH:mm:ss");
   }

   public static LocalDateTime valueOfObj(Object obj) {
      if(obj instanceof Long) {
         return valueOf((Long)((Long)obj));
      } else if(obj instanceof LocalDateTime) {
         return (LocalDateTime)obj;
      } else if(obj instanceof Timestamp) {
         return valueOf((Timestamp)((Timestamp)obj));
      } else if(obj instanceof Date) {
         return toLocalDateTime((Date)obj);
      } else {
         throw new UnsupportedOperationException("obj type:" + obj.getClass());
      }
   }

   public static LocalDateTime valueOf(Long timestamp) {
      if(timestamp.longValue() == 0L) {
         return null;
      } else {
         Instant instant = Instant.ofEpochMilli(timestamp.longValue());
         return instant.atZone(ZoneId.of("Asia/Shanghai")).toLocalDateTime();
      }
   }

   public static LocalDateTime valueOf(Timestamp timestamp) {
      return valueOf((Long)Long.valueOf(timestamp.getTime()));
   }

   public static String format(Date date, String format) {
      return DateUtil.format(date, format);
   }

   public static String formatDateTime(Date date) {
      return DateUtil.formatDateTime(date);
   }

   public static String formatDate(Date date) {
      return DateUtil.formatDate(date);
   }

   public static String formatDateTime(LocalDateTime date) {
      return LocalDateTimeUtil.formatNormal(date);
   }

   public static Date parseDate(CharSequence dateString) {
      return DateUtil.parseDate(dateString);
   }

   public static Date parseDateTime(CharSequence dataString) {
      return DateUtil.parseDateTime(dataString);
   }

   public static LocalDateTime beginOfDay(LocalDateTime date) {
      return LocalDateTimeUtil.beginOfDay(date);
   }

   public static LocalDateTime endOfDay(LocalDateTime date) {
      return LocalDateTimeUtil.endOfDay(date);
   }

   public static void main(String[] args) {
      System.out.println(getFifteenIntervalDate(new Date()));
   }

   public static LocalDateTime beginOfHour(LocalDateTime dateTime) {
      Date date = toDate(dateTime);
      DateTime date1 = DateUtil.beginOfHour(date);
      return toLocalDateTime(date1);
   }

   public static Date beginOfHour(Date dateTime) {
      return DateUtil.beginOfHour(dateTime);
   }

   public static Date beginOfDay(Date dateTime) {
      return DateUtil.beginOfDay(dateTime);
   }

   public static Date endOfDay(Date date) {
      return DateUtil.endOfDay(date);
   }

   public static Date beginOfMonth(Date date) {
      return DateUtil.beginOfMonth(date);
   }

   public static Date endOfMonth(Date date) {
      return DateUtil.endOfMonth(date);
   }

   public static Date endOfHour(Date date) {
      return DateUtil.endOfHour(date);
   }

   public static Date endOfMinute(Date date) {
      return DateUtil.endOfMinute(date);
   }

   public static Date offsetMinute(Date date, int offset) {
      return DateUtil.offsetMinute(date, offset);
   }

   public static Date offsetSecond(Date date, int offset) {
      return DateUtil.offsetSecond(date, offset);
   }

   public static Date offsetHour(Date date, int offset) {
      return DateUtil.offsetHour(date, offset);
   }

   public static Date offsetDay(Date date, int offset) {
      return DateUtil.offsetDay(date, offset);
   }

   public static Date offset(Date date, DateField dateField, int offset) {
      return DateUtil.offset(date, dateField, offset);
   }

   public static Date getTenIntervalDate(Date date) {
      String minuteStr = DateUtil.format(date, "mm");
      Integer minute = Integer.valueOf(minuteStr);

      for(int i = 0; i < minute10Array.length - 1; ++i) {
         if(minute.intValue() >= minute10Array[i].intValue() && minute.intValue() < minute10Array[i + 1].intValue()) {
            String dateFormat = DateUtil.format(date, DatePattern.NORM_DATETIME_FORMAT);
            String newDate = dateFormat.substring(0, 14) + (minute10Array[i].intValue() < 10?"0" + minute10Array[i]:minute10Array[i]) + ":00";
            return DateUtil.parseDateTime(newDate);
         }
      }

      return null;
   }

   public static Date getFifteenIntervalDate(Date date) {
      String minuteStr = DateUtil.format(date, "mm");
      Integer minute = Integer.valueOf(minuteStr);

      for(int i = 0; i < minute15Array.length - 1; ++i) {
         if(minute.intValue() >= minute15Array[i].intValue() && minute.intValue() < minute15Array[i + 1].intValue()) {
            String dateFormat = DateUtil.format(date, DatePattern.NORM_DATETIME_FORMAT);
            String newDate = dateFormat.substring(0, 14) + (minute15Array[i].intValue() < 10?"0" + minute15Array[i]:minute15Array[i]) + ":00";
            return DateUtil.parseDateTime(newDate);
         }
      }

      return null;
   }

   public static Date getThirtyIntervalDate(Date date) {
      String minuteStr = DateUtil.format(date, "mm");
      Integer minute = Integer.valueOf(minuteStr);

      for(int i = 0; i < minute30Array.length - 1; ++i) {
         if(minute.intValue() >= minute30Array[i].intValue() && minute.intValue() < minute30Array[i + 1].intValue()) {
            String dateFormat = DateUtil.format(date, DatePattern.NORM_DATETIME_FORMAT);
            String newDate = dateFormat.substring(0, 14) + (minute30Array[i].intValue() < 10?"0" + minute30Array[i]:minute30Array[i]) + ":00";
            return DateUtil.parseDateTime(newDate);
         }
      }

      return null;
   }

   public static Date getFiveIntervalDate(Date date) {
      String minuteStr = DateUtil.format(date, "mm");
      Integer minute = Integer.valueOf(minuteStr);

      for(int i = 0; i < minute5Array.length - 1; ++i) {
         if(minute.intValue() >= minute5Array[i].intValue() && minute.intValue() < minute5Array[i + 1].intValue()) {
            String dateFormat = DateUtil.format(date, DatePattern.NORM_DATETIME_FORMAT);
            String newDate = dateFormat.substring(0, 14) + (minute5Array[i].intValue() < 10?"0" + minute5Array[i]:minute5Array[i]) + ":00";
            return DateUtil.parseDateTime(newDate);
         }
      }

      return null;
   }

   public static int realDayOfWeek(Date date) {
      int week = DateUtil.dayOfWeek(date);
      int[] dayOfWeeks = new int[]{7, 1, 2, 3, 4, 5, 6};
      return dayOfWeeks[week - 1];
   }

   public static long betweenDay(Date beginDay, Date endDay, boolean isReset) {
      return DateUtil.betweenDay(beginDay, endDay, isReset);
   }

}
